@@ -2,7 +2,7 @@
 
    cd-drive.c: easy to use cd burner software
 
-   Copyright (C) 2002 Red Hat, Inc.
+   Copyright (C) 2002-2004 Red Hat, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -84,6 +84,28 @@ struct CDDrivePriv {
 
 static CDDrive *cd_drive_new (void);
 
+#ifdef USE_HAL
+static LibHalContext *
+get_hal_context (void)
+{
+	static LibHalContext *ctx = NULL;
+	LibHalFunctions hal_functions = {
+		NULL, /* mainloop integration */
+		NULL, /* device_added */
+		NULL, /* device_removed */
+		NULL, /* device_new_capability */
+		NULL, /* property_modified */
+		NULL, /* device_condition */
+	};
+	
+	if (ctx == NULL)
+		ctx = hal_initialize (&hal_functions, FALSE);
+
+	return ctx;
+}
+#endif /* USE_HAL */
+
+
 /* Utility functions, be careful to have a match with what's use in the
  * different bits of code */
 #if !defined (__linux__)
@@ -251,7 +273,7 @@ cd_drive_get_media_type_from_path_full (const char *device, gboolean *is_rewrita
 
 	g_return_val_if_fail (device != NULL, CD_MEDIA_TYPE_ERROR);
 
-	fd = open (device, O_RDONLY|O_EXCL|O_NONBLOCK);
+	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
 	if (fd < 0) {
 		if (errno == EBUSY) {
 			return CD_MEDIA_TYPE_BUSY;
@@ -351,16 +373,8 @@ cd_drive_get_media_type_and_rewritable (CDDrive *cdrom, gboolean *is_rewritable)
 		int num_devices;
 		CDMediaType type;
 		char *hal_type;
-		LibHalFunctions hal_functions = {
-			NULL, /* mainloop integration */
-			NULL, /* device_added */
-			NULL, /* device_removed */
-			NULL, /* device_new_capability */
-			NULL, /* property_modified */
-			NULL, /* device_condition */
-		};
 		
-		ctx = hal_initialize (&hal_functions, FALSE);
+		ctx = get_hal_context ();
 		if (ctx != NULL) {
 			device_names = hal_manager_find_device_string_match (ctx, 
 									     "info.parent",
@@ -413,8 +427,6 @@ cd_drive_get_media_type_and_rewritable (CDDrive *cdrom, gboolean *is_rewritable)
 
 			hal_free_string_array (device_names);
 
-			hal_shutdown (ctx);
-
 			return type;
 		}
 	}
@@ -435,7 +447,7 @@ cd_drive_get_media_size_from_path (const char *device)
 
 	secs = 0;
 
-	fd = open (device, O_RDONLY|O_EXCL|O_NONBLOCK);
+	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
 	if (fd < 0) {
 		if (errno == EBUSY) {
 			return CD_MEDIA_SIZE_BUSY;
@@ -485,20 +497,12 @@ static GList *
 hal_scan (gboolean recorder_only)
 {
 	GList *cdroms = NULL;
-	LibHalFunctions hal_functions = {
-		NULL, /* mainloop integration */
-		NULL, /* device_added */
-		NULL, /* device_removed */
-		NULL, /* device_new_capability */
-		NULL, /* property_modified */
-		NULL, /* device_condition */
-	};
 	int i;
 	int num_devices;
 	char** device_names;
 	LibHalContext *ctx;
 
-	ctx = hal_initialize (&hal_functions, FALSE);
+	ctx = get_hal_context ();
 	if (ctx == NULL) {
 		return NULL;
 	}
@@ -508,7 +512,6 @@ hal_scan (gboolean recorder_only)
 
 	if (device_names == NULL)
 	{
-		hal_shutdown (ctx);
 		return NULL;
 	}
 
@@ -584,7 +587,6 @@ hal_scan (gboolean recorder_only)
 	}
 
 	hal_free_string_array (device_names);
-	hal_shutdown (ctx);
 
 	cdroms = g_list_reverse (cdroms);
 
@@ -752,7 +754,7 @@ get_cd_scsi_id (const char *dev, int *bus, int *id, int *lun)
 	} m_idlun;
 	
 	devfile = g_strdup_printf ("/dev/%s", dev);
-	fd = open(devfile, O_RDONLY | O_NONBLOCK);
+	fd = open(devfile, O_RDWR | O_NONBLOCK);
 	g_free (devfile);
 
 	/* Avoid problems with Valgrind */
@@ -808,7 +810,7 @@ get_device_max_read_speed (char *device)
 
 	max_speed = -1;
 
-	fd = open (device, O_RDONLY|O_EXCL|O_NONBLOCK);
+	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
 	if (fd < 0) {
 		return -1;
 	}
@@ -830,7 +832,7 @@ get_device_max_write_speed (char *device)
 
 	max_speed = -1;
 
-	fd = open (device, O_RDONLY|O_EXCL|O_NONBLOCK);
+	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
 	if (fd < 0) {
 		return -1;
 	}
@@ -1074,7 +1076,7 @@ linux_scan (gboolean recorder_only)
 
 	if (n_scsi_units > 0) {
 		/* open /dev/sg0 to force loading of the sg module if not loaded yet */
-		fd = open ("/dev/sg0", O_RDONLY);
+		fd = open ("/dev/sg0", O_RDWR);
 		if (fd >= 0) {
 			close (fd);
 		}
@@ -1462,6 +1464,61 @@ cd_drive_free (CDDrive *drive)
 	g_free (drive);
 }
 
+gboolean
+cd_drive_lock (CDDrive *drive,
+	       const char *reason,
+	       char **reason_for_failure) 
+{
+	gboolean res;
+
+	if (reason_for_failure != NULL)
+		*reason_for_failure = NULL;
+
+	res = TRUE;
+#ifdef USE_HAL
+	if (drive->priv->udi != NULL) {
+		LibHalContext *ctx;
+		char *dbus_reason;
+		
+		ctx = get_hal_context ();
+		if (ctx != NULL) {
+			res = hal_device_lock (ctx, 
+					       drive->priv->udi,
+					       reason,
+					       &dbus_reason);
+			if (dbus_reason != NULL && 
+			    reason_for_failure != NULL)
+				*reason_for_failure = g_strdup (dbus_reason);
+			if (dbus_reason != NULL)
+				dbus_free (dbus_reason);
+		}
+	}
+#endif
+	return res;
+}
+
+
+gboolean 
+cd_drive_unlock (CDDrive *drive)
+{
+	gboolean res;
+
+	res = TRUE;
+#ifdef USE_HAL
+	if (drive->priv->udi != NULL) {
+		LibHalContext *ctx;
+
+		ctx = get_hal_context ();
+		if (ctx != NULL) {
+			res = hal_device_unlock (ctx, 
+						 drive->priv->udi);
+		}
+	}
+#endif
+	return res;
+}
+
+
 static CDDrive *
 cd_drive_new (void)
 {
@@ -1471,3 +1528,21 @@ cd_drive_new (void)
 	cdrom->priv = g_new0 (CDDrivePriv, 1);
 	return cdrom;
 }
+
+CDDrive *
+cd_drive_copy (CDDrive *drive)
+{
+	CDDrive *cdrom;
+
+	cdrom = cd_drive_new ();
+	cdrom->type = drive->type;
+	cdrom->display_name = g_strdup (drive->display_name);
+	cdrom->max_speed_write = drive->max_speed_write;
+	cdrom->max_speed_read = drive->max_speed_read;
+	cdrom->cdrecord_id = g_strdup (drive->cdrecord_id);
+	cdrom->device = g_strdup (drive->device);
+	cdrom->priv->protocol = drive->priv->protocol;
+	cdrom->priv->udi = g_strdup (drive->priv->udi);
+
+	return cdrom;
+}
